/*
 *      @(#)gtypes.h	1.6     6/10/93
 */

/*
General Types
*/

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

/*
Trace Types
*/

#define Host_ID_Len 80

/* Label types for displayed hosts */
typedef struct Host_Struct{
	char ipnum[Host_ID_Len];	/* Hosts IP num as a string */
	char ipname[Host_ID_Len];	/* Hosts IP name */
	Boolean resolved;		/* Could the host be resolved or not */
	Boolean last_at_site;		/* Is the host the last at the site */	
	float northing;			/* Hosts longitude position in degree-seconds */
	float easting;			/* Hosts longitude position in degree-seconds */
} Host_Struct;


typedef struct Host_Struct_List {
	struct Host_Struct_List *next;
	Host_Struct host;
} host_struct_list, *phost_struct_list;


typedef struct Host_List {
	struct Host_List *next;
	float northing;
	float easting;
} host_list, *phost_list;

#define Label_IPNUM 1
#define Label_IPNAME 2
#define Label_Position 4

typedef struct Trace_Struct{
	int numhosts;			/* Number of gateways in the current trace */
	Boolean label_ipnum;		/* What to label the hosts when they are displayed */
	Boolean label_ipname;
	Boolean label_position;
	int colour;			/* Colour to draw the current trace in */
	struct Host_Struct dest_host;	/* Destination host in the trace */
	struct Host_Struct *hosts;	/* Hosts in the trace */
	struct Trace_Struct *next_trace;	/* Pointer to the next trace */
} Trace_Struct;

#define Max_Trace 1

typedef struct Traces_Status_Struct{
	Boolean tracing;	/* Is a route currently  being traced */
	int numtrace;		/* How many traces are there */
	int tracepid;		/* Process id for the current trace */
	int tracesd;		/* Socket descriptor that is the connection to the trace process */
	Trace_Struct *traces;	/* Trace Structure pointer */
	Boolean visible[Max_Trace]; 	/* Which of the traces are to be displayed */
} Traces_Status_Struct;

/*
Map Display Types
*/

#define INCL_RIV	1
#define INCL_PDY	2
#define INCL_CIT	4
#define INCL_3D		8

/* A structure to hold the current details of the map in the canvas display */
#define Max_Detail_Level 2	/* The maximum map detail that is  catered for */

typedef struct Dimension_Attributes {
	long min_vis;
	long max_vis;
	int min_vis_reg;
	int max_vis_reg;
} Dimension_Attributes;

#define File_Name_Length 80

typedef enum ZoomState_Type {not, zero, one, two} ZoomState_Type;

typedef struct Map_State_Struct{
	int detail; 			/* Level of current map detail */
	char current_map_fn[File_Name_Length];	/* File name of the current map */
	int side_size; 			/* Side size in regions of current map */
	int num_regions; 		/* the side size squared, it doesnt take much room*/
	int region_range; 		/* Range per region */
	long max_range;			/* Range over map */
	int  x_reg_offset; 		/* The number of regions the globe has been rotated - or + */
	int  y_reg_offset; 		/* number of regions the globe has been rotated in a north or south direction */
	unsigned char x_offset; 	/* offset within the region that the globe has been */
	unsigned char y_offset; 	/* offset within the region that the globe has been rotated */
	float rotation_fraction; 	/* The percentage the map is to be rotated */
	float zoom; 			/* A value indicating how far the map has been zoomed in on */
	unsigned long includes; 	/* bit flags for includes such as cities, rivers... */
	char rot_dir;			/* holds the direction of the continuous rotation */
	Boolean rotating;		/* is the roation function on? */
	long rotation;			/* amount of rotation */
	Dimension_Attributes x;
	Dimension_Attributes y;	
} Map_State_Struct; 

/* Structure that describes the vectors in a particular region of a map */
typedef struct Region_Struct {
	int nvect;
	int *nseg;
	unsigned char **line;
} Region_Struct;

/* Structure that describes the coordinates for a bounding box, used in zooming actions */
typedef struct Bounding_Box_Struct {
	int origx;
	int origy;
	int startx;
	int starty;
	int currentx;
	int currenty;
	int finalx;
	int finaly;
} Bounding_Box_Struct;

struct city_struct {
         char name[ 80];
         float northing;
         char ndirect;
         float easting;
         char edirect;
} city;

typedef struct city_struct_list {
	struct city_struct city;
	struct city_struct_list *next;
} city_list, *pcity_list;

typedef struct Zoom_Undo_Stack {
	struct Zoom_Undo_Stack *next;
	struct Zoom_Undo_Stack *prev;
	Map_State_Struct map_state;
} zoom_undo_stack, *pzoom_undo_stack;
