#include "geotraceman.h"
#include <sys/wait.h>
#include "items.h"
/*
Returns the pointer to the specified trace number
*/
void 
go_to_trace(trace_number, tracep)
	int trace_number;
	Trace_Struct **tracep;
{
	int t; /* Loop counter */
	
	*tracep = ts.traces;

	for(t=0; t < (trace_number - 1); t++) 
		*tracep = (*tracep)->next_trace;
}	

/*
Forks off a process to actually do the trace
*/ 
int /* Returns the process id */
fork_trace (dest_host_ipnum, ipnum_fh)
	char dest_host_ipnum[Host_ID_Len];
	int *ipnum_fh;
{

	int ipnum_pipe[2];

	int pid;

	int i;

	i = pipe(ipnum_pipe);
      
#ifdef OSF1
	fcntl(ipnum_pipe[0], F_SETFL, FNONBIO);   
#else
	fcntl(ipnum_pipe[0], F_SETFL, O_NDELAY);  /* say dont block the reads */
#endif

	/* fork the trace_route process and run in child */
	if ((pid = fork()) == 0)
	{
		close(ipnum_pipe[0]); 
		trace_route(dest_host_ipnum, ipnum_pipe[1]);   
		close(ipnum_pipe[1]);
		ts.tracepid = -1;
		exit(0);
	}
   
	close(ipnum_pipe[1]); 

	*ipnum_fh = ipnum_pipe[0];

	return(pid);
}

Boolean readtracepipeanddraw(pipefh)
	int pipefh;
{
	int bytes_read; /* Loop counter */
	Host_Struct *lhost, *chost;
	Trace_Struct *ctrace;
	char ipnum[Host_ID_Len];
	Boolean first_host = FALSE;
	int delay, hopcount, tot_delay;
	char *c;
	Widget tmpWidget;
	char sig_size;

	go_to_trace(ts.numtrace, &ctrace);

	if ((bytes_read = read(pipefh, ipnum, 80)) == -1) {
		return(False);
	}

/* this is a craig hack */
        if (bytes_read == 0) 
                return(False);


	/* process data coming from the pipe */
	if(strcmp(ipnum,"finished") == 0) {
		close(ts.tracesd); 
		ts.tracing = False;
if(!batchmode)
{
	        if ((tmpWidget = XtNameToWidget( toplevel, "*tstatusnLabel")) == NULL)
        	        fail0("No such widget tstatusnLabel");
	        setLabel(tmpWidget, "finished");
}
		if (ctrace->numhosts == 0) {
			DisplayMessage(toplevel, "Error", "No routers were encountered");
			return True;
		}
		(*ctrace).hosts[(ctrace->numhosts - 1)].last_at_site = True;
		return(True);
	}

	c=ipnum;
	while(*c != ',')    /* get to the first comma */
		c++;
		
        *c = '\0';
	/* move past the comma and get the trace statistics passed */
	sscanf(++c,"%d,%d,%d", &delay, &hopcount, &tot_delay);

if (!batchmode)
{
	/* update the trace statistics widgets */
        if ((tmpWidget = XtNameToWidget( toplevel, "*numhostsnLabel")) == NULL)
                fail0("No such widget numhostnLabel");
	sprintf(c, "%d", hopcount);
        setLabel(tmpWidget, c);

        if ((tmpWidget = XtNameToWidget( toplevel, "*lasthostnLabel")) == NULL)
                fail0("No such widget lasthostnLabel");
	sprintf(c, "%d ms", delay);
        setLabel(tmpWidget, c);

        if ((tmpWidget = XtNameToWidget( toplevel, "*totaltimenLabel")) ==
NULL)
                fail0("No such widget totaltimenLabel");
        sprintf(c, "%d ms", tot_delay);
        setLabel(tmpWidget, c);
}


	if (ctrace->numhosts != 0) {
		if (strcmp(ipnum,(*ctrace).hosts[(ctrace->numhosts - 1)].ipnum) == 0)
			return(False);			
	}

	if (create_host(ctrace) == 1)
		exit(1);

	if (ctrace->numhosts == 1)
		first_host = TRUE;

	chost = &((*ctrace).hosts[(ctrace->numhosts - 1)]);

	if (first_host != TRUE)
		lhost = &((*ctrace).hosts[(ctrace->numhosts) - 2]);

	strcpy(chost->ipnum, ipnum);
	
	resolve_host(chost, &sig_size);

	if ((chost->resolved == FALSE) && (first_host == FALSE)) {
		chost->easting = (*ctrace).hosts[(ctrace->numhosts - 2)].easting;
		chost->northing = (*ctrace).hosts[(ctrace->numhosts - 2)].northing;
	}

	if (chost->ipnum[0] != -1) {
		printf("%d: %s, %s, %3.2fn : %3.2fe Resolved: %d\n",(*ctrace).numhosts, chost->ipname, ipnum, (chost->northing/3600), (chost->easting/3600), chost->resolved);

		display_extra_host(ctrace, first_host, sig_size);
if (!batchmode)
		swapbuffers();

	}
	return(False);
}

/*
Create the space for an extra host, if the first then use malloc, otherwise use realloc 
*/
int
create_host(tracep)
	Trace_Struct *tracep;
{
	int memory_size;

	if ((*tracep).numhosts == 0) {
		memory_size = sizeof(Host_Struct);	
		(*tracep).hosts = (Host_Struct *) malloc(memory_size);
	}
	else {
		memory_size = sizeof(Host_Struct) * ((*tracep).numhosts + 1);
		(*tracep).hosts = (Host_Struct *) realloc( ((char *) (*tracep).hosts), memory_size);
	}


	if ((*tracep).hosts == NULL) {
		fprintf(stderr,"GTM: Error allocating space for host in trace.\n");
		return(1);
	}

	/* Increment the number of hosts */
	tracep->numhosts++;

	/* Initialise host values */
	(*tracep).hosts[(tracep->numhosts - 1)].resolved = FALSE;
	(*tracep).hosts[(tracep->numhosts - 1)].last_at_site = FALSE;
	(*tracep).hosts[(tracep->numhosts - 1)].ipnum[0] = NULL;
	(*tracep).hosts[(tracep->numhosts - 1)].ipname[0] = NULL;
	(*tracep).hosts[(tracep->numhosts - 1)].northing = 0.0;
	(*tracep).hosts[(tracep->numhosts - 1)].easting = 0.0;
	
/*fprintf(stderr,"Host %d, %s\n",tracep->numhosts,tracep->hosts[tracep->numhosts].ipnum);*/
	return(0);
}

/*
Create the space for a new trace, given the pointer to it
*/
int /* Zero on success one on failure */
create_trace(tracepp)
Trace_Struct **tracepp;
{

	*tracepp = (Trace_Struct *) malloc(sizeof(Trace_Struct));
        if (*tracepp == NULL) {
                return(1);
        }

	/* Initialise the number of hosts in the trace to zero */
	(*tracepp)->numhosts = 0;
	(*tracepp)->label_position = False;
	(*tracepp)->label_ipname = True;
	(*tracepp)->label_ipnum = False;
if (!batchmode)
	setTick(ITEM_LABEL_IPNAME);


	/* Increment the number of traces */
	ts.numtrace++;
	ts.visible[(ts.numtrace - 1)] = TRUE;	
	return(0);
}
